package com.durant.kafka.service;

import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

/**
 * @author 冯凯
 * @version 1.0
 * @description: kafka的service
 * @date 2023/11/25 13:57
 */
@Service
@Log4j2
public class KafkaService {

    @Autowired
    private  Producer<String,String> producer;


    /**
     * 发送消息到Kafka
     *
     * @param topic   消息的主题
     * @param key     消息的键
     * @param value   消息的值
     * @return        返回消息的元数据
     */
    public RecordMetadata send(String topic, String key, String value){
        log.info("kafka发送消息为:"+value);
        Future<RecordMetadata> result = producer.send(new ProducerRecord<>(topic,0, key, value));
        RecordMetadata recordMetadata=null;
        try {
            recordMetadata=result.get();
        } catch (InterruptedException | ExecutionException e) {
            throw new RuntimeException(e);
        }
        return recordMetadata;
    }



}
