package com.durant.mqtt.rabbit;

import com.alibaba.fastjson.JSONObject;
import com.durant.kafka.constant.KafkaConstants;
import com.durant.mqtt.config.MqttClientConnect;
import com.durant.mqtt.config.MqttConfig;
import com.durant.mqtt.service.MqttService;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.UUID;

/**
 * @author 冯凯
 * @version 1.0
 * @description:
 * @date 2023/11/29 15:48
 */
@Component
@Log4j2
public class SubscriberRabbit {
    @Autowired
    private MqttClientConnect mqttClientConnect;
    @Autowired
    private MqttConfig mqttConfig;
    /**
     *注入rabbit
     */
    @Autowired
    private RabbitTemplate rabbitTemplate;
    @Autowired
    private MqttService mqttService;
    @RabbitListener(queuesToDeclare={@Queue(value = "sub_top")})
    public void consumerSubscribe(String mesg, Message message, Channel channel){
        log.info("收到需要订阅的主题是:"+mesg);
        // 创建一个MqttClient对象并返回
        MqttClient mqttClient;
        try {
            log.info("开始连接:"+System.currentTimeMillis());
            // 连接MQTT代理服务器
            mqttClient= new MqttClient(mqttConfig.getBroker(), UUID.randomUUID().toString(),new MemoryPersistence());
            // 创建MqttConnectOptions对象
            MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();
            mqttConnectOptions.setUserName("worker");
            mqttConnectOptions.setPassword("worker".toCharArray());
            mqttConnectOptions.setConnectionTimeout(60);
            mqttConnectOptions.setKeepAliveInterval(60);
            // 连接MQTT代理服务器
            mqttClient.connect();
            //通过消息队列告知kafka消费者准备好要拉取哪个主题下的哪个分区的消息
            KafkaConstants kafkaConstants = KafkaConstants.getBuilder("topA", "0");
            rabbitTemplate.convertAndSend("kafka_top", JSONObject.toJSONString(kafkaConstants),msg->{
                msg.getMessageProperties().setMessageId(UUID.randomUUID().toString().replaceAll("-",""));
                return msg;
            });
            // 设置回调函数
            mqttClient.setCallback(mqttService);
            // 订阅指定的MQTT主题
            mqttClient.subscribe(mesg,0);

            // 订阅指定的MQTT主题
            log.info("准备订阅主题是:"+mesg);
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
        log.info("连接成功:"+System.currentTimeMillis());
    }

}
